"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastify_1 = __importDefault(require("fastify"));
const fastify_plugin_1 = require("../fastify-plugin");
const fixtures_1 = require("./fixtures");
describe('fastify-plugin', () => {
    let app;
    beforeEach(async () => {
        app = (0, fastify_1.default)();
    });
    afterAll(async () => {
        await app.close();
    });
    it('should forward requests', async () => {
        const handler = jest.fn();
        const payload = (0, fixtures_1.createPubSubRequest)('forward me');
        app.register(fastify_plugin_1.pubSubFastifyPlugin, { handler });
        const res = await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(res.statusCode).toBe(204);
        expect(handler).toHaveBeenCalled();
        // TODO: Get context value
        // expect(handler).toHaveBeenCalledWith({
        //   message: payload.message,
        //   data: 'forward me',
        //   context: JSON.parse(JSON.stringify(payload)),
        // });
    });
    it('should forward statusCode', async () => {
        const handler = jest.fn(() => ({ statusCode: 404 }));
        const payload = (0, fixtures_1.createPubSubRequest)('forward me');
        app.register(fastify_plugin_1.pubSubFastifyPlugin, { handler });
        const res = await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(res.statusCode).toBe(404);
        expect(handler).toHaveBeenCalled();
        // TODO: Get context value
        // expect(handler).toHaveBeenCalledWith({
        //   message: payload.message,
        //   data: 'forward me',
        //   context: JSON.parse(JSON.stringify(payload)),
        // });
    });
    it('should return onError on throw', async () => {
        const handler = () => {
            throw new Error('error');
        };
        const onError = jest.fn();
        const payload = (0, fixtures_1.createPubSubRequest)('forward me');
        app.register(fastify_plugin_1.pubSubFastifyPlugin, { handler, onError });
        await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(onError).toHaveBeenCalledWith(new Error('error'), expect.any(Object));
    });
    it('should not return onError on throw', async () => {
        const handler = () => {
            throw new Error('error');
        };
        const payload = (0, fixtures_1.createPubSubRequest)('forward me');
        app.register(fastify_plugin_1.pubSubFastifyPlugin, { handler });
        const res = await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(res.payload).toMatchInlineSnapshot(`"{"statusCode":500,"error":"Internal Server Error","message":"error"}"`);
    });
    it('should throw when parsing fails', async () => {
        const handler = jest.fn(() => ({ hello: 'world' }));
        const parser = jest.fn(() => {
            throw new Error('parsing fail');
        });
        const payload = (0, fixtures_1.createPubSubRequest)('forward me');
        app.register(fastify_plugin_1.pubSubFastifyPlugin, {
            handler,
            parser,
        });
        const res = await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(parser).toHaveBeenCalled();
        expect(res.payload).toMatchInlineSnapshot(`"{"statusCode":500,"error":"Internal Server Error","message":"parsing fail"}"`);
    });
    it('should not throw if request is missing attributes', async () => {
        const handler = jest.fn();
        const payload = {
            message: {
                data: 'ImZvcndhcmQgbWUi',
                messageId: 'in dolor Ut',
            },
            subscription: 'mollit sint',
        };
        app.register(fastify_plugin_1.pubSubFastifyPlugin, { handler });
        const res = await app.inject({
            method: 'POST',
            url: '/',
            payload,
        });
        expect(res.statusCode).toBe(204);
    });
});
//# sourceMappingURL=fastify-plugin.test.js.map